--02.12.2016
-- strategy as follows
-- 1) compute list of random canonical curves of given genus and given charakteristic
-- 2) compute all betti tables via "minimalBetti"
-- 3) catch all curves with a certian Betti number beta_{p,p+1} and
--     collect them in a new list
-- 4) run the test for finite field Green for all curves in this class 
  

-- the function strand picks all non-zero Betti numbers in the 1st linear strand
strand = method()
strand(BettiTally) := B -> (
     str:=select(apply(1..#B,i->B_(i,{i+1},i+1)),b->b!=0);
     str)

--picks the Bettinumber b_(pos,pos+1) in the linear strand
pickBetti=method()
pickBetti(BettiTally,ZZ):=(B,pos)->( (strand(B))_(pos-1))
pickBetti(BettiTally):= (B)->(
    pos:=#strand(B);
    pickBetti(B,pos))

-- given a list of Betti tables BettiL
-- the following function selects all indices such that the last values of
-- in the linear strand are asgiven by the list B
pickCandidates = method()
pickCandidates(List,ZZ,ZZ) := (BettiL, b,pos) ->( 
    select(#BettiL,i-pickBetti(BettiL_i,pos)==b)
    )
pickCandidates(List,ZZ) := (BettiL, b) ->( 
    select(#BettiL,i->pickBetti(BettiL_i)==b)
    )
pickCandidates(List,List) := (BettiL, B) ->(
    i:=0;
    BL:=pickCandidates(BettiL,B_(#B-1-i));
    if #(unique apply(BL,i->#strand(BettiL_i)))=!=1 then error"length differ";  
    s:=#strand(BettiL_(BL_0));  
    while i<(#B-1) do (
     	i=i+1;
    	pos=s-i;
        BL=select(BL,j->pickBetti(BettiL_j,pos)==B_(#B-1-i));	
	); BL) 
 
 -- the following function assumes that we know that the curve is a hot candidate
-- other wise for testing the Betties one should use "minimalBetti"

-- the function tests the finite field Green conjecture 
-- INPUT: the ideal of a canonical curve
testRGC=method(Options=>{Printing=>false}) 
testRGC(Ideal):=opt->IC->(
--GC_INITIAL_HEAP_SIZE... needed for bigger examples
  T:= ring IC;
  if opt.Printing then (print("-- computing non-minimal resolution"));
  time betti(sresIC:=res(IC,FastNonminimal=>true));
  if opt.Printing then (print("-- computing minimal Betti numbers"));
  time bsres:=betti(sresIC,Minimize=>true);
--bsres=BettiL_(candidates_0)
--    	bsres=BettiL_(candidates_0)
  str:=select(apply(1..#B,i->B_(i,{i+1},i+1)),b->b!=0);-- the Betties in the linear strand
  if opt.Printing then  (print("-- partially minimizing the resolution"));
   mm:=#str;-- the critical homological degree
  if (mm==ceiling((genus IC-3)/2)) then (return true);-- testing if the curve as extra syzygies at all
    --The matrix we want to minimize:
  betti (M1:=submatrixByDegrees(sresIC.dd_(mm-1),mm,mm));
  M1kk:=lift(submatrixByDegrees(sresIC.dd_(mm),mm+1,mm+1) , coefficientRing ring M1);
   -- speed improvement idea by mike, since the following is too slow
   -- kerM1kk= ker M1kk;-- still slow
  --time ns:= nullSpace mutableMatrix(M1kk);
  time mns:=matrix( nullSpace mutableMatrix(M1kk) );
  syzM:=map(T^{rank target mns:-mm-1}, T^{rank source mns:-mm-1},sub(mns, T));
-- now we can compute the relevant matrix in the linear strand:
  if opt.Printing then  (print("-- computing the relevant map in the linear strand"));
  time betti (M:=submatrixByDegrees(sresIC.dd_(mm),mm,mm+1)*syzM);
  if opt.Printing then (print("-- testing if dim coker M==0"));
  Mt:= transpose M; 
  time  if( dim (coker Mt)==0) then (print("==> dim ann coker ==0") and (return true)); -- if ==0 then finite length
  if opt.Printing then (print("--computing codim ann coker transpose M"));
  --time    annCokM:= ann (coker transpose M,Strategy=>Quotient);
  --   if (degree annCokM==codim annCokM+1)then (print("==> ann coker is a scroll") and (return true));  -- is it a scroll?
  --   return (degree annCokM, dim T-codim annCokM)
  cokMt:=coker Mt;
   time codimCokMt:=codim cokMt;
   degCokMt:= degree cokMt;
   if (degCokMt==codimCokMt+1)then (print("==> ann coker is a scroll") and (return true));  -- is it a scroll?  
      print("(degree,dim ):");
   return (degCokMt, dim T-codimCokMt)
     ) 

-- the following function also checks the RGC but uses, that the Bettitable is known in advance

testRGC(Ideal,BettiTally):=opt->(IC,B)->(
--GC_INITIAL_HEAP_SIZE... needed for bigger examples
  T:= ring IC;
  if opt.Printing then (print("-- computing non-minimal resolution"));
  time betti(sresIC:=res(IC,FastNonminimal=>true));
  --if opt.Printing then (print("-- computing minimal Betti numbers"));
  --time bsres:=betti(sresIC,Minimize=>true);
  bsres:=B;
--bsres=BettiL_(candidates_0)
--    	bsres=BettiL_(candidates_0)
  str:=select(apply(1..#B,i->B_(i,{i+1},i+1)),b->b!=0);-- the Betties in the linear strand
  if opt.Printing then  (print("-- partially minimizing the resolution"));
   mm:=#str;-- the critical homological degree
  if (mm==ceiling((genus IC-3)/2)) then (return true);-- testing if the curve as extra syzygies at all
    --The matrix we want to minimize:
  betti (M1:=submatrixByDegrees(sresIC.dd_(mm-1),mm,mm));
  M1kk:=lift(submatrixByDegrees(sresIC.dd_(mm),mm+1,mm+1) , coefficientRing ring M1);
   -- speed improvement idea by mike, since the following is too slow
   -- kerM1kk= ker M1kk;-- still slow
  --time ns:= nullSpace mutableMatrix(M1kk);
  time mns:=matrix( nullSpace mutableMatrix(M1kk) );
  syzM:=map(T^{rank target mns:-mm-1}, T^{rank source mns:-mm-1},sub(mns, T));
-- now we can compute the relevant matrix in the linear strand:
  if opt.Printing then  (print("-- computing the relevant map in the linear strand"));
  time betti (M:=submatrixByDegrees(sresIC.dd_(mm),mm,mm+1)*syzM);
  if opt.Printing then (print("-- testing if dim coker M==0"));
  Mt:= transpose M; 
  time  if( dim (coker Mt)==0) then (print("==> dim ann coker ==0") and (return true)); -- if ==0 then finite length
  if opt.Printing then (print("--computing codim ann coker transpose M"));
  --time    annCokM:= ann (coker transpose M,Strategy=>Quotient);
  --   if (degree annCokM==codim annCokM+1)then (print("==> ann coker is a scroll") and (return true));  -- is it a scroll?
  --   return (degree annCokM, dim T-codim annCokM)
  cokMt:=coker Mt;
   time codimCokMt:=codim cokMt;
   degCokMt:= degree cokMt;
   if (degCokMt==codimCokMt+1)then (print("==> ann coker is a scroll") and (return true));  -- is it a scroll?  
   print("(degree,dim ):");
   return (degCokMt, dim T-codimCokMt)
     ) 

-- the following function does the same as the RGC function additionaly 
-- it computes the degree of the annihilator in any case. 
-- If we just want to check the refined Green conjecture one should not use
-- this function
testRGC2=method(Options=>{Printing=>false})
testRGC2(Ideal,BettiTally):=opt->(IC,B)->(
--GC_INITIAL_HEAP_SIZE... needed for bigger examples
  T:= ring IC;
  if opt.Printing then (print("-- computing non-minimal resolution"));
  time betti(sresIC:=res(IC,FastNonminimal=>true));
  --if opt.Printing then (print("-- computing minimal Betti numbers"));
  --time bsres:=betti(sresIC,Minimize=>true);
  bsres:=B;
--bsres=BettiL_(candidates_0)
--    	bsres=BettiL_(candidates_0)
  str:=select(apply(1..#B,i->B_(i,{i+1},i+1)),b->b!=0);-- the Betties in the linear strand
  if opt.Printing then  (print("-- partially minimizing the resolution"));
   mm:=#str;-- the critical homological degree
  if (mm==ceiling((genus IC-3)/2)) then (return true);-- testing if the curve as extra syzygies at all
    --The matrix we want to minimize:
  betti (M1:=submatrixByDegrees(sresIC.dd_(mm-1),mm,mm));
  M1kk:=lift(submatrixByDegrees(sresIC.dd_(mm),mm+1,mm+1) , coefficientRing ring M1);
   -- speed improvement idea by mike, since the following is too slow
   -- kerM1kk= ker M1kk;-- still slow
  --time ns:= nullSpace mutableMatrix(M1kk);
  time mns:=matrix( nullSpace mutableMatrix(M1kk) );
  syzM:=map(T^{rank target mns:-mm-1}, T^{rank source mns:-mm-1},sub(mns, T));
-- now we can compute the relevant matrix in the linear strand:
  if opt.Printing then  (print("-- computing the relevant map in the linear strand"));
  time betti (M:=submatrixByDegrees(sresIC.dd_(mm),mm,mm+1)*syzM);
  if opt.Printing then (print("-- testing if dim coker M==0"));
  Mt:= transpose M; 
  --time  if( dim (coker Mt)==0) then (print("==> dim ann coker ==0") and (return true)); -- if ==0 then finite length
  if opt.Printing then (print("--computing codim ann coker transpose M"));
  --time    annCokM:= ann (coker transpose M,Strategy=>Quotient);
  --   if (degree annCokM==codim annCokM+1)then (print("==> ann coker is a scroll") and (return true));  -- is it a scroll?
  --   return (degree annCokM, dim T-codim annCokM)
  cokMt:=coker Mt;
   time codimCokMt:=codim cokMt;
   degCokMt:= degree cokMt;
   --if (degCokMt==codimCokMt+1)then (print("==> ann coker is a scroll") and (return true));  -- is it a scroll?  
   print("(degree,dim ):");
   return (degCokMt, dim T-codimCokMt)
     ) 

end;
--------------------------------------------------------------------
--==================================================================
--------------------------------------------------------------------
restart
loadPackage("RandomCurvesOverVerySmallFiniteFields",Reload=>true)
load("TestRefinedGreen.m2")

-- edit the folowing line:
(g,p)=(9,3)-- the genus and characteristic

-- loading the list of examples:

filename="examples_g_"|toString(g)|"_char_"|toString(p)   
filePath="producedExamples/g"|toString(g)|"/"|toString(filename)
kk=ZZ/p
T=kk[t_0..t_(g-1)]
L= value get filePath;
#L
BettiPath="producedExamples/g"|toString(g)|"/bettiTables_"|toString(filename)
BettiL=value get BettiPath;
#BettiL
-- we pick a candidate
tally BettiL
candidates=pickCandidates(BettiL,390)
IC=L_(candidates_0);
B=BettiL_(candidates_0)
--time testRGC2(IC,B,Printing=>true)

time testRGC(IC,B, Printing=>true)
time testRGC(IC, Printing=>true)


restart
uninstallPackage("RandomCurvesOverVerySmallFiniteFields")
installPackage("RandomCurvesOverVerySmallFiniteFields")
